﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using Infragistics.Win;
using Infragistics.Win.UltraWinGrid;

namespace TestGrid
{
    public partial class Form1 : Form, IUIElementDrawFilter
    {
        /// <summary>
        /// Index of row clicked.
        /// </summary>
        private int _rowIndex = -1;
        /// <summary>
        /// Index of column clicked.
        /// </summary>
        private int _colIndex = -1;
        /// <summary>
        /// Flag to check the mouse click.
        /// </summary>
        private bool _isEnteringEditMode = true;

        private System.Drawing.Brush _fillBrush;

        /// <summary>
        /// Constant to denote mouse left button down.
        /// </summary>
        private const int MOUSEEVENTF_LEFTDOWN = 0x02;
        /// <summary>
        /// Constant to denote mouse left button up.
        /// </summary>
        private const int MOUSEEVENTF_LEFTUP = 0x04;
        /// <summary>
        /// Mouse event.
        /// </summary>
        /// <param name="dwFlags">Specifies various aspects of mouse 
        /// motion and button clicking.</param>
        /// <param name="dx">Specifies the mouse's absolute 
        /// position along the x-axis</param>
        /// <param name="dy">Specifies the mouse's absolute position 
        /// along the y-axis</param>
        /// <param name="cButtons">Specifies the 
        /// amount of wheel movement.</param>
        /// <param name="dwExtraInfo">Specifies an additional 
        /// value associated with the mouse event.</param>
        [DllImport("user32.dll", CharSet = CharSet.Auto, CallingConvention = CallingConvention.StdCall)]
        public static extern void mouse_event(long dwFlags, long dx, long dy, long cButtons, long dwExtraInfo);

        public Form1()
        {
            InitializeComponent();

            this.ultraGrid1.DisplayLayout.Bands[0].Override.RowSelectors =
                DefaultableBoolean.True;
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            DataTable dt = new DataTable();
            dt.Columns.Add("Col1");
            dt.Columns.Add("Col2");
            dt.Columns.Add("Col3");
            dt.Columns.Add("Col4");
            DataRow dr;
            int count = 0;
            for (int i = 0; i < 10; ++i)
            {
                dr = dt.NewRow();
                count = 0;
                foreach (DataColumn col in dt.Columns)
                {
                    dr[col] = i.ToString() + count.ToString();
                    count++;
                }
                dt.Rows.Add(dr);
            }
            dt.AcceptChanges();
            ultraGrid1.DataSource = dt;

            this.ultraGrid1.DrawFilter = this;
            this._fillBrush = new Pen(Color.White).Brush;
        }

        private void ultraGrid1_BeforeEnterEditMode(object sender, CancelEventArgs e)
        {
            e.Cancel = this._isEnteringEditMode;
        }

        #region IUIElementDrawFilter Members
        /// <summary>
        /// Function to cancel the drawing active cell if no selected row.
        /// </summary>
        /// <param name="drawPhase">When the drawing takes place</param>
        /// <param name="drawParams">Draw Parameter</param>
        /// <returns>True if drawn else false</returns>
        /// <remarks>
        /// Method Name		:	DrawElement
        /// Author			:	Ceaser Y
        /// Created Date	:	24-Oct-2008
        /// </remarks>
        public bool DrawElement(DrawPhase drawPhase, ref UIElementDrawParams drawParams)
        {
            //// This will only be called for the BeforeDrawBackColor phase of a  
            //// EditorWithTextDisplayTextUIElement based on the 
            //flags returned from GetPhasesToFilter.  
            bool isDrawn = false;
            if (ultraGrid1.Selected.Rows.Count == 0)
            {
                Rectangle elementRect = drawParams.Element.Parent.Rect;
                // Draw a the rectangular area of the parent control.
                drawParams.Graphics.FillRectangle(this._fillBrush, elementRect);
                // Return true to prevent the element from drawing normally.
                isDrawn = true;
            }
            return isDrawn;
        }
        /// <summary>
        /// Function to get the drawing phase.
        /// </summary>
        /// <param name="drawParams">Draw Parameter</param>
        /// <returns>DrawPhase of the element</returns>
        /// <remarks>
        /// Method Name		:	GetPhasesToFilter
        /// Author			:	Ceaser Y
        /// Created Date	:	24-Oct-2008
        /// </remarks>
        public DrawPhase GetPhasesToFilter(ref UIElementDrawParams drawParams)
        {
            // If the element being drawn is a EditorWithTextDisplayTextUIElement
            //Then its Back color drawing is only considered.            
            Infragistics.Win.DrawPhase drawPhase = DrawPhase.None;
            if (drawParams.Element is EditorWithTextDisplayTextUIElement)
            {
                //if (_isCancelDraw)
                {
                    drawPhase = Infragistics.Win.DrawPhase.BeforeDrawBackColor;
                }
            }
            return drawPhase;
        }
        #endregion

        private void btnCheck_Click(object sender, EventArgs e)
        {

        }

        private void ultraGrid1_AfterRowActivate(object sender, EventArgs e)
        {

        }

        private void ultraGrid1_AfterCellActivate(object sender, EventArgs e)
        {
            ultraGrid1.ActiveCell.Activated = true;
        }

        private void ultraGrid1_MouseDown(object sender, MouseEventArgs e)
        {
            try
            {
                //Declare and retrieve a reference to the UIElement
                Infragistics.Win.UIElement aUIElement;
                aUIElement = ultraGrid1.DisplayLayout.UIElement.
                    ElementFromPoint(new Point(e.X, e.Y));
                //Declare and retrieve a reference to the Cell
                UltraGridCell aCell;
                RowSelectorUIElement rsElement;
                aCell = aUIElement.GetContext(
                    typeof(UltraGridCell)) as UltraGridCell;
                if (aCell != null)
                {
                    if (e.Button == MouseButtons.Right)
                    {
                        ultraGrid1.Selected.Rows.Clear();
                    }
                    aCell.Row.Activate();
                    ultraGrid1.PerformAction(UltraGridAction.ToggleRowSel);
                    if (_rowIndex == aCell.Row.Index &&
                        _colIndex == aCell.Column.Index &&
                        e.Button == MouseButtons.Left)
                    {
                        this._isEnteringEditMode = false;
                        aCell.Activate();
                        aCell.Activation = Activation.AllowEdit;
                        this.ultraGrid1.PerformAction(UltraGridAction.EnterEditMode);
                        if (aCell.Style ==
                            Infragistics.Win.UltraWinGrid.ColumnStyle.DropDownList
                            || aCell.Column.Style ==
                            Infragistics.Win.UltraWinGrid.ColumnStyle.DropDownList)
                        {
                            this.ultraGrid1.PerformAction(
                                UltraGridAction.ToggleDropdown);
                        }
                        this._isEnteringEditMode = true;
                    }
                    if (e.Button == MouseButtons.Left)
                    {
                        _rowIndex = aCell.Row.Index;
                        _colIndex = aCell.Column.Index;
                    }
                }
                rsElement = aUIElement.GetAncestor(
                    typeof(RowSelectorUIElement)) as RowSelectorUIElement;
                if (rsElement != null)
                {
                    if (e.Button == MouseButtons.Right)
                    {
                        rsElement.Row.Activate();
                        if (!rsElement.Row.Selected)
                        {
                            ultraGrid1.PerformAction(
                                UltraGridAction.ToggleRowSel);
                        }
                    }
                }
            }
            catch (Exception exp)
            {
               
            }
        }

        private void ultraGrid1_KeyDown(object sender, KeyEventArgs e)
        {
            try
            {
                if (e.KeyCode == Keys.Enter)
                {
                    //Activate the cell which is below the current below.
                    if (ultraGrid1.ActiveCell != null)
                    {
                        this.ultraGrid1.PerformAction(
                            UltraGridAction.BelowCell);
                    }
                }
                else if (e.KeyCode == Keys.F2)
                {
                    UltraGridCell aCell = ultraGrid1.ActiveCell;
                    if (aCell != null)
                    {
                        Point point;
                        Point ptCurr;
                        int xCord;
                        int yCord;
                        _rowIndex = aCell.Row.Index;
                        _colIndex = aCell.Column.Index;
                        point = ultraGrid1.PointToScreen(new Point(0, 0));
                        xCord = aCell.GetUIElement().Rect.X;
                        yCord = aCell.GetUIElement().Rect.Y;
                        xCord += point.X;
                        yCord += point.Y;
                        ptCurr = Cursor.Position;
                        Cursor.Position = new Point(xCord, yCord);
                        mouse_event(MOUSEEVENTF_LEFTDOWN | MOUSEEVENTF_LEFTUP,
                            xCord, yCord, 0, 0);
                        Cursor.Position = ptCurr;
                    }
                }
            }
            catch (Exception exp)
            {
                
            }
        }

        private void ultraGrid1_BeforeCellDeactivate(object sender, CancelEventArgs e)
        {

        }
    }
}
